@@ -1,6 +1,6 @@
 /*
     connection.c -- connection list management
-    Copyright (C) 2000-2009 Guus Sliepen <guus@tinc-vpn.org>,
+    Copyright (C) 2000-2012 Guus Sliepen <guus@tinc-vpn.org>,
                   2000-2005 Ivo Timmermans
                   2008      Max Rijevski <maksuf@gmail.com>
 
@@ -29,7 +29,7 @@
 #include "xalloc.h"
 
 avl_tree_t *connection_tree;	/* Meta connections */
-connection_t *broadcast;
+connection_t *everyone;
 
 static int connection_compare(const connection_t *a, const connection_t *b) {
 	return a < b ? -1 : a == b ? 0 : 1;
@@ -37,14 +37,14 @@ static int connection_compare(const connection_t *a, const connection_t *b) {
 
 void init_connections(void) {
 	connection_tree = avl_alloc_tree((avl_compare_t) connection_compare, (avl_action_t) free_connection);
-	broadcast = new_connection();
-	broadcast->name = xstrdup("everyone");
-	broadcast->hostname = xstrdup("BROADCAST");
+	everyone = new_connection();
+	everyone->name = xstrdup("everyone");
+	everyone->hostname = xstrdup("BROADCAST");
 }
 
 void exit_connections(void) {
 	avl_delete_tree(connection_tree);
-	free_connection(broadcast);
+	free_connection(everyone);
 }
 
 connection_t *new_connection(void) {
@@ -60,44 +60,54 @@ connection_t *new_connection(void) {
 	return c;
 }
 
-void free_connection(connection_t *c) {
-	if(c->name)
-		free(c->name);
+void free_connection_partially(connection_t *c) {
+	free(c->inkey);
+	free(c->outkey);
+	free(c->mychallenge);
+	free(c->hischallenge);
+	free(c->outbuf);
 
-	if(c->hostname)
-		free(c->hostname);
+	c->inkey = NULL;
+	c->outkey = NULL;
+	c->mychallenge = NULL;
+	c->hischallenge = NULL;
+	c->outbuf = NULL;
 
-	if(c->inkey)
-		free(c->inkey);
-
-	if(c->outkey)
-		free(c->outkey);
+	c->buflen = 0;
+	c->reqlen = 0;
+	c->tcplen = 0;
+	c->allow_request = 0;
+	c->outbuflen = 0;
+	c->outbufsize = 0;
+	c->outbufstart = 0;
 
 	if(c->inctx) {
 		EVP_CIPHER_CTX_cleanup(c->inctx);
 		free(c->inctx);
+		c->inctx = NULL;
 	}
 
 	if(c->outctx) {
 		EVP_CIPHER_CTX_cleanup(c->outctx);
 		free(c->outctx);
+		c->outctx = NULL;
 	}
 
-	if(c->mychallenge)
-		free(c->mychallenge);
+	if(c->rsa_key) {
+		RSA_free(c->rsa_key);
+		c->rsa_key = NULL;
+	}
+}
 
-	if(c->hischallenge)
-		free(c->hischallenge);
+void free_connection(connection_t *c) {
+	free_connection_partially(c);
+
+	free(c->name);
+	free(c->hostname);
 
 	if(c->config_tree)
 		exit_configuration(&c->config_tree);
 
-	if(c->outbuf)
-		free(c->outbuf);
-
-	if(c->rsa_key)
-		RSA_free(c->rsa_key);
-
 	free(c);
 }
 
