@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #ifndef __TINC_SYSTEM_H__
@@ -31,10 +31,6 @@ typedef int bool;
 #define false 0
 #endif
 
-/* Include localisation support */
-
-#include "gettext.h"
-
 #ifndef HAVE_STRSIGNAL
 # define strsignal(p) ""
 #endif
